/** Custom App Styles */

/* Rows */
.row-equal {
    display: flex;
    flex-wrap: wrap;
    }
    .row-equal .col {
        margin-left: 0 !important;
    }

/* Tables */
table { background: #fff; }

/* Cards */
.card-add {
    background: transparent;
    border: 2px dashed #b0bec5; /** blue-grey lighten-3 **/
    display: flex;
    align-items: center;
    justify-content: center;
    margin: .5rem 0 1rem 0;
    height: calc(100% - 2*(.5rem + 2px));
    cursor: pointer;
    color: #b0bec5;
}

/* Logs */
.collapsible li.waiting .collapsible-header { border-left: 2px solid #ffc107;} /** amber **/
.collapsible li.success .collapsible-header { border-left: 2px solid #4caf50;} /** green **/
.collapsible li.error .collapsible-header { border-left: 2px solid #f44336;} /** red **/
.log {
    padding: .5rem;
    overflow-y: scroll;
    max-height: 64vh;
}
.log pre {
    white-space: pre-wrap;
    word-break: break-all;
}

/* Nav headers */
.nav-extended .nav-content i.material-icons {
    line-height: 36px;
    height: 36px;
}

.experiment-meta > span {
    display: block;
    margin-bottom: .125rem;
}