"""Provides Config Holder and Config Loader modules to easily load/store config items.

Import :py:mod:`.Loader` and :py:class:`.Config`
so that they can be easly imported by other packages/modules.
"""
# flake8: noqa
from .Config import Config
from .Loader import Loader
