{% extends "layout.jinja" %}
{% block title %}Page Not Found{% endblock %}
{% block content %}
  <h1>Page Not Found</h1>
  <p>{{ msg }}</p>
  <p><a href="{{ url_for('dashboard.dashboard') }}">go somewhere nice</a></p>
{% endblock %}